//
//  PREqualize.h
//  PRICE
//
//  Created by Riccardo Mottola on Fri Dec 05 2003.
//  Copyright (c) 2003 Carduus. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify it under the terms of the version 2 of the GNU General Public License as published by the Free Software Foundation.
// This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

#import <AppKit/AppKit.h>
#import "PRImage.h"

#define COLOR_SPACE_RGB 1
#define COLOR_SPACE_YUV 2


@interface PREqualize : NSObject {

}

- (PRImage *)equalizeImage:(PRImage *)srcImage :(int)space;

@end
