//
//  PRCDFTLowPass.h
//  PRICE
//  DFT based LowPass filter controller
//
//  Created by Riccardo Mottola on Sat Sep 13 2003.
//  Copyright (c) 2003 Carduus. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify it under the terms of the version 2 of the GNU General Public License as published by the Free Software Foundation.
// This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

#import <AppKit/AppKit.h>



@interface PRCDFTLowPass : NSObject {
    IBOutlet NSWindow *filterWindow;
    IBOutlet NSButton *autoRangeCheck;
    IBOutlet NSSlider *passBandSlider;
    IBOutlet NSSlider *stopBandSlider;
    IBOutlet NSTextField *passBandVal;
    IBOutlet NSTextField *passBandValPi;
    IBOutlet NSTextField *stopBandVal;
    IBOutlet NSTextField *stopBandValPi;
    float passBandFreq;
    float stopBandFreq;
    BOOL autoRange;
}

- (IBAction)showDFTLP:(id)sender;
- (IBAction)dftLPOK:(id)sender;
- (IBAction)dftLPCancel:(id)sender;
- (IBAction)dftLPAutoRange:(id)sender;
- (IBAction)changeStopBand:(id)sender;
- (IBAction)changePassBand:(id)sender;

@end
