//
//  PRCBriCon.h
//  PRICE
//
//  Created by Riccardo Mottola on Thu Mar 3 2005.
//  Copyright (c) 2005 Carduus. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify it under the terms of the version 2 of the GNU General Public License as published by the Free Software Foundation.
// This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

#import <AppKit/AppKit.h>

@interface PRCBriCon : NSObject
{
    IBOutlet NSWindow    *briconWindow;
    IBOutlet NSStepper   *briStep;
    IBOutlet NSSlider    *briSlider;
    IBOutlet NSTextField *briVal;
    IBOutlet NSStepper   *conStep;
    IBOutlet NSSlider    *conSlider;
    IBOutlet NSTextField *conVal;
}

- (IBAction)showBriCon:(id)sender;
- (IBAction)briconOk:(id)sender;
- (IBAction)briconCancel:(id)sender;
- (IBAction)briconReset:(id)sender;
- (IBAction)changeBri:(id)sender;
- (IBAction)changeCon:(id)sender;

@end

