//
//  PRCEqualize.h
//  PRICE
//  Image Equalization Controller
//
//  Created by Riccardo Mottola on Fri Dec 05 2003.
//  Copyright (c) 2003 Carduus. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify it under the terms of the version 2 of the GNU General Public License as published by the Free Software Foundation.
// This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

#import <AppKit/AppKit.h>


@interface PRCEqualize : NSObject {
    IBOutlet NSWindow *equalWindow;
    IBOutlet NSPopUpButton *colorSpaceChoice;
}

- (IBAction)showEqualize:(id)sender;
- (IBAction)equalizeOK:(id)sender;
- (IBAction)equalizeCancel:(id)sender;

@end
