//
//  PRCHisto.h
//  PRICE
//  Histogram Controller
//
//  Created by Riccardo Mottola on Thu Dec 18 2003.
//  Copyright (c) 2003 Carduus. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify it under the terms of the version 2 of the GNU General Public License as published by the Free Software Foundation.
// This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

#import "PRCHistogram.h"
#import "PRHisto.h"


@interface PRCHisto : PRCHistogram
{
    IBOutlet PRHisto *histoView;
}

@end
